<?php
use CRM_Loginsecurity_ExtensionUtil as E;

class CRM_Loginsecurity_BAO_LoginSecurityDevice extends CRM_Loginsecurity_DAO_LoginSecurityDevice {

  /**
   * Create a new LoginSecurityDevice based on array-data
   *
   * @param array $params key-value pairs
   * @return CRM_Loginsecurity_DAO_LoginSecurityDevice|NULL
   *
  public static function create($params) {
    $className = 'CRM_Loginsecurity_DAO_LoginSecurityDevice';
    $entityName = 'LoginSecurityDevice';
    $hook = empty($params['id']) ? 'create' : 'edit';

    CRM_Utils_Hook::pre($hook, $entityName, CRM_Utils_Array::value('id', $params), $params);
    $instance = new $className();
    $instance->copyValues($params);
    $instance->save();
    CRM_Utils_Hook::post($hook, $entityName, $instance->id, $instance);

    return $instance;
  } */

  /**
   * Updates the current session (if it has been more than 15 minutes), logging the IP and fingerprint.
   *
   * Returns the contact_id if it is a new/updated session (used to determine whether to load the fingerprinting JS).
   */
  public static function logCurrentSession() {
    // Check if the device is already registered
    $session_id = session_id();
    $ip_address = CRM_Utils_System::ipAddress();
    $contact_id = CRM_Core_Session::getLoggedInContactID();
    $user_agent = $_SERVER['HTTP_USER_AGENT'] ?? 'unknown';
    $fingerprint = NULL;

    // Probably drush
    if (!$contact_id) {
      if (!empty($_REQUEST['api_key'])) {
        $contact_id = CRM_Core_DAO::singleValueQuery('SELECT id FROM civicrm_contact WHERE api_key = %1', [
          1 => [$_REQUEST['api_key'], 'String'],
        ]);

        // We force this, because otherwise we will have a lot of records
        $session_id = 'REST API';
        $fingerprint = 'REST API';
      }

      if (!$contact_id) {
        if (!empty($_REQUEST['cs'])) {
          $userChecksum = CRM_Utils_Request::retrieveValue('cs', 'String');
          $temp_cid = CRM_Utils_Request::retrieveValue('cid', 'Positive');
          $validUser = CRM_Contact_BAO_Contact_Utils::validChecksum($temp_cid, $userChecksum);

          if ($validUser) {
            $contact_id = $temp_cid;
          }
        }

        if (!$contact_id) {
          return FALSE;
        }
      }
    }

    // Cleanup the user agent
    $user_agent = str_replace('Mozilla/5.0 ', '', $user_agent);
    $user_agent = str_replace(' (KHTML, like Gecko)', '', $user_agent);
    $user_agent = str_replace('(', '', $user_agent);
    $user_agent = str_replace(')', '', $user_agent);

    $dao = CRM_Core_DAO::executeQuery('SELECT *, UNIX_TIMESTAMP(last_seen) as last_seen_ts
      FROM civicrm_login_security_device
      WHERE contact_id = %1 AND ip_address = %2 AND session_id = %3
      ORDER BY id ASC LIMIT 1', [
      1 => [$contact_id, 'Positive'],
      2 => [$ip_address, 'String'],
      3 => [$session_id, 'String'],
    ]);

    if ($dao->fetch()) {
      // Update the last_seen every 15 mins
      $now = time();
      $update_delay = 900;

      if ($dao->last_seen_ts - $now > $update_delay) {
        CRM_Core_DAO::executeQuery('UPDATE civicrm_login_security_device SET last_seen = NOW(), user_agent = %3 WHERE contact_id = %1 AND ip_address = %2', [
          1 => [$contact_id, 'Positive'],
          2 => [$ip_address, 'String'],
          3 => [$user_agent, 'String'],
        ]);

        return $contact_id;
      }
    }
    else {
      $api = \Civi\Api4\LoginSecurityDevice::create(FALSE)
        ->addValue('contact_id', $contact_id)
        ->addValue('ip_address', $ip_address)
        ->addValue('session_id', $session_id)
        ->addValue('user_agent', $user_agent);

      if ($fingerprint) {
        $api->addValue('fingerprint', $fingerprint);
      }

      $api->execute();

      return $contact_id;
    }

    return FALSE;
  }

  /**
   * Returns the GeoIP record information for a given IP.
   */
  public static function getGeoIP(String $ip) {
    // @todo Setting?
    $db_country = __DIR__ . '/../../../db/country.mmdb';
    $db_city = __DIR__ . '/../../../db/city.mmdb';

    // Avoid fatals if the files are not there
    if (empty(\Civi::$statics[__CLASS__]['autoload'])) {
      if (file_exists(__DIR__ . '/../../../vendor/autoload.php') && file_exists($db_country)) {
        require_once __DIR__ . '/../../../vendor/autoload.php';
        \Civi::$statics[__CLASS__]['autoload'] = 'success';
      }
      else {
        Civi::log()->warning('loginsecurity: the composer dependencies are missing. If you installed from git, please run "composer install" from the extension directory.');
        \Civi::$statics[__CLASS__]['autoload'] = 'failed';
      }
    }

    if (\Civi::$statics[__CLASS__]['autoload'] != 'success') {
      return null;
    }

    // Exceptions where the CiviCRM country does not match the MaxMind country
    // By using CiviCRM's country name, we can send the name to 'ts' for translation
    $country_civi_map = [
      'Russia' => 'Russian Federation',
    ];

    try {
      // @todo Make this more configurable, ex: if using a paid database?
      $reader = new \GeoIp2\Database\Reader($db_country);
      $record = $reader->country($ip);
      $country_name = $record->country->name;

      if (isset($country_civi_map[$country_name])) {
        $country_name = $country_civi_map[$country_name];
      }

      $country_name = ts($country_name, ['context' => 'country']);

      if (file_exists($db_city)) {
        $reader = new \GeoIp2\Database\Reader($db_city);
        $record = $reader->city($ip);
        $city_name = $record->city->name;

        if ($city_name) {
          return $city_name . ', ' . $country_name;
        }
      }

      return $country_name;
    }
    catch (Exception $e) {
      Civi::log()->warning('loginsecurity: Reader Exception: ' . $e->getMessage());
      return null;
    }

    return null;
  }

}
